// 时间戳转换为年月日时分秒
export const timesTampChange = (timestamp, format = 'Y-M-D h:m:s') => {
  if (!timestamp) return ''
  const date = new Date(timestamp)
  const Y = date.getFullYear().toString()
  const M = (date.getMonth() + 1).toString().padStart(2, '0')
  const D = date.getDate().toString().padStart(2, '0')

  const h = date.getHours().toString().padStart(2, '0')
  const m = date.getMinutes().toString().padStart(2, '0')
  const s = date.getSeconds().toString().padStart(2, '0')

  const formatter = { 'Y': Y, 'M': M, 'D': D, 'h': h, 'm': m, 's': s } // 替换format中的占位符
  let formattedDate = format.replace(/Y|M|D|h|m|s/g, matched => formatter[matched]) // 使用正则表达式匹配并替换占位符
  
  if (!formattedDate) formattedDate = Y + '-' + M + '-' + D + ' ' + h + ':' + m + ':' + s

  return formattedDate
}

// 将 Wed May 01 2024 00:00:00 GMT+0800 (中国标准时间) 转换为时间戳
export const getTimeStamp = (str) => {
  if (!str) return ''
  const date = new Date(str)
  return date.getTime()
}

// 传入一个时间戳返回这个日期的最早时间点以及最晚时间点 输出：[1721232000000, 1721318399999]
export const getDayBounds = (timestamp) => {
  const date = new Date(timestamp)
  date.setHours(0, 0, 0, 0)
  const startOfDay = date.getTime()
  const endOfDay = new Date(timestamp)
  endOfDay.setHours(23, 59, 59, 999)
  if (endOfDay.getDate() !== date.getDate()) {
    endOfDay.setDate(endOfDay.getDate() - 1)
    endOfDay.setHours(23, 59, 59, 999)
  }
  // 返回包含最早和最晚时间点的时间戳的数组
  return [startOfDay, endOfDay.getTime()]
}

// 传入 Wed May 01 2024 00:00:00 GMT+0800 (中国标准时间) 输出 [2024-07-18 00:00:00, 2024-07-18 23:59:59]
export const getStartAndEndOfDay = (dateTimeStr) => {
  const date = new Date(dateTimeStr)

  const year = date.getFullYear()
  const month = String(date.getMonth() + 1).padStart(2, '0')
  const day = String(date.getDate()).padStart(2, '0')

  const startTime = `${year}-${month}-${day} 00:00:00`
  const endTime = `${year}-${month}-${day} 23:59:59`
  return [startTime, endTime]
}

// 传入一组时间戳，返回 [最早时间点，最晚时间点]
export const  convertTimestampsToDayRange = (timestamps) => {
  if (timestamps.length < 2) {
    throw new Error('Timestamps array must contain at least two elements')
  }
  function formatDate(date) {
    const year = date.getFullYear();
    const month = String(date.getMonth() + 1).padStart(2, '0')
    const day = String(date.getDate()).padStart(2, '0')
    const hours = String(date.getHours()).padStart(2, '0')
    const minutes = String(date.getMinutes()).padStart(2, '0')
    const seconds = String(date.getSeconds()).padStart(2, '0')
    return `${year}-${month}-${day} ${hours}:${minutes}:${seconds}`
  }

  const startDate = new Date(timestamps[0])
  const endDate = new Date(timestamps[1])

  const startOfDay = new Date(startDate.getFullYear(), startDate.getMonth(), startDate.getDate())

  const endOfDay = new Date(endDate.getFullYear(), endDate.getMonth(), endDate.getDate() + 1, 0, 0, 0, -1)

  return [formatDate(startOfDay), formatDate(endOfDay)]
}

/**
 * 时间日期转换
 * @param {dayjs.ConfigType} date 当前时间，new Date() 格式
 * @param {string} format 需要转换的时间格式字符串
 * @description format 字符串随意，如 `YYYY-mm、YYYY-mm-dd`
 * @description format 季度："YYYY-mm-dd HH:MM:SS QQQQ"
 * @description format 星期："YYYY-mm-dd HH:MM:SS WWW"
 * @description format 几周："YYYY-mm-dd HH:MM:SS ZZZ"
 * @description format 季度 + 星期 + 几周："YYYY-mm-dd HH:MM:SS WWW QQQQ ZZZ"
 * @returns {string} 返回拼接后的时间字符串
 */
import dayjs from 'dayjs';
export function formatDate(date, format = 'YYYY-MM-DD HH:mm:ss') {
  // 日期不存在，则返回空
  if (!date) {
    return '';
  }
  // 日期存在，则进行格式化
  if (format === undefined) {
    format = 'YYYY-MM-DD HH:mm:ss';
  }
  return dayjs(date).format(format);
}

// 设置面试邀请默认时间
export const getInterviewInviteDefaultTime = () => {
  const today = new Date()
  today.setDate(today.getDate() + 1)

  const time = new Date(today.getFullYear(), today.getMonth(), today.getDate(), 10, 0, 0)
  return {
    time,
    timeStamp: time.getTime()
  }
}

// 返回时间差，最小计量单位为一个月。示例用法：
// const startTime = 1633072800000; // 2021-10-01 00:00:00 UTC（秒级时间戳）
// const endTime = 1667244000000;  // 2022-11-01 00:00:00 UTC（秒级时间戳）
export const getTimeDifferenceInChinese = (startTime, endTime) => {
  // 将时间戳转换为 Date 对象（假设时间戳单位为毫秒）
  const startDate = startTime ? new Date(startTime) : new Date();
  const endDate = endTime ? new Date(endTime) : new Date();

  // 计算年份差
  let yearsDiff = endDate.getFullYear() - startDate.getFullYear();
  // 计算月份差（考虑年份差后调整）
  let monthsDiff = endDate.getMonth() - startDate.getMonth();
  // 如果月份差为负，则从上一年借月
  if (monthsDiff < 0) {
    yearsDiff--;
    monthsDiff += 12;
  }
  // 计算日期差（考虑月份差后调整，如果日期差为负，则从上一月借天）
  let daysDiff = endDate.getDate() - startDate.getDate();
  if (daysDiff < 0) {
    monthsDiff--;
    // 获取 startDate 所在月的最后一天
    const lastDayOfMonth = new Date(startDate.getFullYear(), startDate.getMonth() + 1, 0).getDate();
    daysDiff += lastDayOfMonth; // 加上最后一天以补全月份差
  }

  // 构建结果字符串
  let result = "";
  if (yearsDiff > 0) {
    result += `${yearsDiff}年`;
  }
  if (monthsDiff > 0) {
    if (result) {
      // 如果已经有年份差异，则直接添加月份数（不带单位），后面正则替换会处理
      result += monthsDiff;
    } else {
      // 如果没有年份差异，则正常添加月份和单位
      result += `${monthsDiff}个月`;
      // 特别处理只有1个月的情况
      if (monthsDiff === 1) {
        result = "不到1个月"; // 直接替换为“不到1个月”，避免后续复杂的正则替换
      }
    }
  } else if (!result && daysDiff >= 0) {
    // 如果没有年份和月份差异，但天数差异存在（这里其实没处理天数，只是为完整性添加）
    // 理论上应该处理天数，但题目要求只到月份，所以这里直接返回“不到1个月”
    result = "不到1个月";
  }
 
  // 如果之前添加了月份数但没有年份（且不是直接处理的1个月情况），则需要去除末尾多余的数字并添加“个月”
  if (result && !/\d年$/.test(result) && /\d$/.test(result)) {
    result += "个月";
  }
  return result
}
